package com.haochuan.hciptvbasic.util;

import android.app.Activity;
import android.content.Context;
import android.opengl.GLSurfaceView;
import android.util.Log;
import android.view.Surface;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.Toast;

import com.asha.vrlib.MD360Director;
import com.asha.vrlib.MD360DirectorFactory;
import com.asha.vrlib.MDVRLibrary;
import com.asha.vrlib.model.BarrelDistortionConfig;
import com.asha.vrlib.model.MDPinchConfig;
import com.haochuan.core.BaseMediaPlayer;
import com.haochuan.core.Logger;

import static com.haochuan.core.util.MessageCode.PLAYER_OBJ_NULL;

/**
 * @author: xulin
 * @projectName:HcIPTVBasic
 * @createTime:2020/6/5 16:23
 */
public class VRUtil {
    GLSurfaceView glSurfaceView;

    public MDVRLibrary createVRLibrary(Context context, BaseMediaPlayer mMediaPlayerWrapper) {
        //初始化GLSurfaceView，并将它添加到view树中
        glSurfaceView = new GLSurfaceView(context);
        FrameLayout.LayoutParams params = new FrameLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT);
        glSurfaceView.setLayoutParams(params);

        if (glSurfaceView.getParent() == null) {
            Activity activity = (Activity)context;
            ViewGroup viewGroup = (ViewGroup)activity.getWindow().getDecorView();
            viewGroup.addView(glSurfaceView, 0);
        }
        return MDVRLibrary.with(context)
                .displayMode(MDVRLibrary.DISPLAY_MODE_NORMAL)
                .interactiveMode(MDVRLibrary.INTERACTIVE_MODE_KEYBOARD)
                .asVideo(new MDVRLibrary.IOnSurfaceReadyCallback() {
                    @Override
                    public void onSurfaceReady(Surface surface) {
                        Logger.d("onSurfaceReady");
                        mMediaPlayerWrapper.setSurface(surface);
                    }
                })
                .ifNotSupport(new MDVRLibrary.INotSupportCallback() {
                    @Override
                    public void onNotSupport(int mode) {
                        String tip = mode == MDVRLibrary.INTERACTIVE_MODE_MOTION
                                ? "onNotSupport:MOTION" : "onNotSupport:" + String.valueOf(mode);
                        Logger.d("onNotSupport:MOTION,onNotSupport:"+ String.valueOf(mode));
                    }
                })
                .pinchConfig(new MDPinchConfig().setMin(1.0f).setMax(8.0f).setDefaultValue(0.1f))
                .pinchEnabled(true)
                .directorFactory(new MD360DirectorFactory() {
                    @Override
                    public MD360Director createDirector(int index) {
                        return MD360Director.builder().setPitch(90).build();
                    }
                })
                .projectionFactory(new CustomProjectionFactory())
                .barrelDistortionConfig(new BarrelDistortionConfig().setDefaultEnabled(false).setScale(0.95f))
                .build(glSurfaceView);
    }

    public void release(Activity activity){
        if(glSurfaceView != null && glSurfaceView.getParent() != null){
            ViewGroup viewGroup = (ViewGroup) activity.getWindow().getDecorView();
            viewGroup.removeView(glSurfaceView);
        }else{
            Logger.e(PLAYER_OBJ_NULL,"播放器对象为null,不能调用destroyVideo函数");
        }
    }
}
